package com.purui.ghip.filter;

import com.purui.ghip.config.AuthConfig;

import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;

/**
 * 鉴权
 *
 * @author charlie
 * @date 2024/9/9
 */
@Component
public class AuthFilter extends AbstractGatewayFilterFactory<AuthConfig> {

	public AuthFilter() {
		super(AuthConfig.class);
	}

	@Override
	public GatewayFilter apply(AuthConfig config) {
		return (exchange, chain) -> {
			ServerHttpRequest request = exchange.getRequest();
			HttpHeaders headers = request.getHeaders();

			// 获取header
			String authHeader = headers.getFirst(HttpHeaders.AUTHORIZATION);

			if (authHeader == null || !authHeader.startsWith("Bearer ")) {
				ServerHttpResponse response = exchange.getResponse();
				response.setStatusCode(HttpStatus.UNAUTHORIZED);
				return response.setComplete();
			}
			authHeader = authHeader.substring(7);
			String token = config.getToken();

			if (authHeader != null && authHeader.equals(token)) {
				return chain.filter(exchange);
			}

			// 鉴权失败，返回 401 Unauthorized
			ServerHttpResponse response = exchange.getResponse();
			response.setStatusCode(HttpStatus.UNAUTHORIZED);
			return response.setComplete();
		};
	}

}
